#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm>
using namespace std;

#include "TROOT.h"
#include "TSystem.h"
#include "TStyle.h"
#include "TString.h"
#include "TFile.h"
#include "TTree.h"
#include "TCanvas.h"

#include "TDatime.h"

TString dir = "./";
TString filename_base = "20240121";
Int_t n_rawfile = 1;

TString temp_rh_csv_name = "20240121_temp_rh.csv";
typedef pair<Float_t, Float_t> MyPair;
typedef map<UInt_t, MyPair> MyMap;
MyMap temp_rh_map{};

void build_temp_rh_map(TString csv_name)
{
	ifstream inFile(csv_name);
	string strline, strtmp;
	while (getline(inFile, strline)) {
		Float_t temp = 0, rh = 0;
		TDatime datime;
		stringstream ss(strline);

		getline(ss, strtmp, ',');
		temp = (Float_t)(((TString)strtmp).Atof());
		getline(ss, strtmp, ',');
		rh = (Float_t)(((TString)strtmp).Atof());
		MyPair temp_rh = make_pair(temp, rh);

		getline(ss, strtmp, ',');
		datime.Set(strtmp.c_str());
		UInt_t key = datime.Convert() / 60;

		temp_rh_map[key] = temp_rh;
	}
}

MyPair get_temp_rh_from_map(TDatime datime)
{
	MyPair result = make_pair(0, 0);
	UInt_t key = datime.Convert() / 60;
	MyMap::iterator iter;
	iter = temp_rh_map.find(key);
	if (iter != temp_rh_map.end()) {
		result = iter->second;
	} else {
		for (size_t i = 1; i < 7; i++) {
			iter = temp_rh_map.find(key - i);
			if (iter != temp_rh_map.end()) {
				result = iter->second;
				break;
			}
			iter = temp_rh_map.find(key + i);
			if (iter != temp_rh_map.end()) {
				result = iter->second;
				break;
			}
		}
	}
	return result;
}

void read_a_txt(TString filefullpath, TTree* tree)
{
	Int_t ev_num;
	TDatime* datime;
	Float_t temperature, rh;
	auto waves = new Float_t[8][1024]{};
	tree->SetBranchAddress("ev", &ev_num);
	tree->SetBranchAddress("datime", &datime);
	tree->SetBranchAddress("temp", &temperature);
	tree->SetBranchAddress("rh", &rh);
	tree->SetBranchAddress("waves", waves);

	ifstream inFile(filefullpath.Data());
	string strline;
	string tmp, strtime;
	while (getline(inFile, strline)) {
		// Event:	42	V1742	0x00110000	2024/01/21-21:07:22	Group:	1	nCH:	8	Corrected:	1	Calibrated:	1
		string firstword = "empty";
		stringstream ss(strline);
		ss >> firstword;
		if (firstword == "empty") break;
		ss >> ev_num >> tmp >> tmp >> strtime;
		string sqltime = strtime;
		replace(sqltime.begin(), sqltime.end(), '-', ' ');
		replace(sqltime.begin(), sqltime.end(), '/', '-');

		datime->Set(sqltime.c_str());

		for (size_t ch = 0; ch < 8; ch++) {
			getline(inFile, strline);
			stringstream ss(strline);
			Float_t ch_id;
			ss >> ch_id;
			for (size_t i = 0; i < 1024; i++) {
				ss >> waves[ch][i];
			}
		}

		MyPair temp_rh = get_temp_rh_from_map(*datime);
		temperature = temp_rh.first;
		rh = temp_rh.second;
		tree->Fill();

        //////////////////////////////////////////////////////
		// the "if" condition is always "false", this "if" should be meaningless
        // however, the program will break without this "cout" in the lopp
        ///////////////////////////////////////////////////////
		if (ev_num + 1 == 0) {
			cout << ev_num + 1 << ends;
		}

	}
	delete[] waves;
}

void assemble_data(TString rootfilename)
{
	cout << "Start Preparing" << endl;
	Int_t ev_num;
	TDatime datime;
	Float_t temperature, rh;
	auto waves = new Float_t[8][1024]{};

	TFile* rawfile = new TFile(rootfilename, "recreate");
	TTree* tree_raw = new TTree("raw", "raw data of the experiment");
	tree_raw->SetDirectory(rawfile);
	tree_raw->Branch("ev", &ev_num, "ev/I");
	tree_raw->Branch("datime", &datime);
	tree_raw->Branch("temp", &temperature, "temp/F");
	tree_raw->Branch("rh", &rh, "rh/F");
	tree_raw->Branch("waves", waves, "waves[8][1024]/F");

	for (Int_t i_file = 0; i_file < n_rawfile; i_file++) {
		TString filefullpath = dir + filename_base + "_" + TString::Itoa(i_file, 10) + ".txt";
		read_a_txt(filefullpath, tree_raw);
	}

	rawfile->Write();

	delete[] waves;
	delete tree_raw;
	delete rawfile;
}

void test()
{
	build_temp_rh_map(dir + temp_rh_csv_name);

	assemble_data(dir + filename_base + "_raw.root");
}