#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm>
using namespace std;

#include "TROOT.h"
#include "TSystem.h"
#include "TStyle.h"
#include "TString.h"
#include "TFile.h"
#include "TTree.h"
#include "TCanvas.h"

#include "TDatime.h"

typedef enum {
	Amplitude = 0b01,
	Integral = 0b10,
	Both = 0b11
} ThresholdStyle;
Int_t baseline_range_low = 25;
Int_t baseline_range_high = 425;
Int_t peak_range_low = 430;
Int_t peak_range_high = 830;

void select_data(TString rawfilename, TString outfilename, ThresholdStyle style, Double_t threshold1, Double_t threshold2)
{
	cout << "Start Analysing" << endl;
	Int_t ev_num;
	TDatime datime;
	TDatime* pdatime;
	Float_t temperature, rh;
	auto waves = new Float_t[8][1024]{};
	auto signals = new Float_t[4][1024]{};
	Float_t baseline[4]{};
	Float_t amplitude[4]{};
	Float_t integral[4]{};

	TFile* rawfile = new TFile(rawfilename, "read");
	TTree* tree_raw = rawfile->Get<TTree>("raw");
	tree_raw->SetBranchAddress("ev", &ev_num);
	tree_raw->SetBranchAddress("datime", &pdatime);
	tree_raw->SetBranchAddress("temp", &temperature);
	tree_raw->SetBranchAddress("rh", &rh);
	tree_raw->SetBranchAddress("waves", waves);

	TFile* shrinkedfile = new TFile(outfilename, "recreate");
	TTree* tree = new TTree("shrinked", "shrinked data of the experiment");
	tree->SetDirectory(shrinkedfile);
	tree->Branch("ev", &ev_num, "ev/I");
	tree->Branch("datime", &datime);
	tree->Branch("temp", &temperature, "temp/F");
	tree->Branch("rh", &rh, "rh/F");
	tree->Branch("signals", signals, "signals[4][1024]/F");
	tree->Branch("baseline", baseline, "baseline[4]/F");
	tree->Branch("amp", amplitude, "amp[4]/F");
	tree->Branch("integral", integral, "integral[4]/F");

	Long64_t n_event = tree_raw->GetEntries();
	for (Long64_t i = 0; i < n_event; i++) {
		tree_raw->GetEntry(i);

		pdatime->Copy(datime);

		for (int j = 0; j < 4; j++) {
			baseline[j] = 0;
			amplitude[j] = 0;
			integral[j] = 0;
			for (int k = 0; k < 1024; k++) {
				signals[j][k] = waves[2 * j][k] - waves[2 * j + 1][k];
			}
			for (int k = baseline_range_low; k < baseline_range_high; k++) {
				baseline[j] += signals[j][k];
			}
			baseline[j] /= baseline_range_high - baseline_range_low;
			for (int k = 0; k < 1024; k++) {
				signals[j][k] -= baseline[j];
			}
			amplitude[j] = *max_element(signals[j] + peak_range_low, signals[j] + peak_range_high);
			for (int k = peak_range_low; k < peak_range_high; k++) {
				integral[j] += signals[j][k];
			}
		}

		Bool_t is_valid = true;
		if (style & ThresholdStyle::Amplitude) {
			for (int j = 0; j < 4; j++) {
				if (amplitude[j] < threshold1) {
					is_valid = false;
					break;
				}
			}
		}
		if (style & ThresholdStyle::Integral) {
			for (int j = 0; j < 4; j++) {
				if (integral[j] < threshold2) {
					is_valid = false;
					break;
				}
			}
		}

		if (is_valid) {
			tree->Fill();
		}

		//////////////////////////////////////////////////////
		// the "if" condition is always "false", this "if" should be meaningless
        // however, the program will break without this "cout" in the lopp
        ///////////////////////////////////////////////////////
		if (ev_num + 1 == 0) {
			cout << ev_num + 1 << ends;
		}
	}

	shrinkedfile->Write();
	rawfile->Close();

	delete[] waves;
	delete[] signals;
	delete tree;
	delete rawfile;
	delete shrinkedfile;
}

void test1()
{
	select_data("./20240121_raw.root", "./20240121_select.root", Both, 4, 400);
}